PUZZLE_TEMPLATE_ID = "1drSoyrE4gM3JaGweDkOybwXWdKPIDTfUmB1gQCYS3Uw"
PUZZLE_TEMPLATE_SHEETS = ["Text", "Square grid", "Hex Grid",
                          "Formula reference: indexing"]

def sheets_create(turb, name):
    """Create a new spreadsheet with the given name.

    Returns a dict with 'id' and 'url' of the spreadsheet
    """

    spreadsheet = create_spreadsheet(turb, name)

    return {
        'id': spreadsheet['spreadsheetId'],
        'url': spreadsheet['spreadsheetUrl']
    }

def create_spreadsheet(turb, name):
    """
    Returns the request's dict which has at least the following keys:

        ['spreadsheetId']: ID for this spreadsheet
        ['spreadsheetUrl']: URL of this spreadsheet
        ['sheets'][0]['properties']['sheetId']: ID of first sheet inside
    """

    # Create a new spreadsheet
    spreadsheet_body = {
        'properties': {
            'title': name
        }
    }

    spreadsheet = turb.sheets.create(body=spreadsheet_body).execute()

    # Now that we've created a new spreadsheet, we need to also allow
    # anyone with the link to the sheet to be able to edit it.
    turb.permissions.create(fileId=spreadsheet["spreadsheetId"],
                            body={'type': 'anyone', 'role': 'writer'},
                            fields='id').execute()

    return spreadsheet

def sheets_create_for_puzzle(turb, puzzle):
    """Creates a new sheet for a puzzle of the given name

    Like sheets_create(), but also copies the puzzle template sheet.

    Here, 'puzzle' is a dict that must have a 'name' key and may optionally
    have a 'channel_url' or 'url' key.
    """

    # First create the new spreadsheet
    spreadsheet = create_spreadsheet(turb, puzzle['name'])
    spreadsheet_id = spreadsheet['spreadsheetId']

    # Then, copy some useful sheets over from the Template spreadsheet

    response = turb.sheets.get(spreadsheetId=PUZZLE_TEMPLATE_ID).execute()

    for sheet in response["sheets"]:
        if sheet["properties"]["title"] in PUZZLE_TEMPLATE_SHEETS:
            res = turb.sheets.sheets().copyTo(
                spreadsheetId=PUZZLE_TEMPLATE_ID,
                sheetId=sheet["properties"]["sheetId"],
                body={
                    "destinationSpreadsheetId": spreadsheet_id
                }).execute()
            # Rename each copied sheet to match the name from the template
            sheet_name = sheet["properties"]["title"]
            # Except for "Text" which we rename to the puzzle name
            if sheet_name == "Text":
                sheet_name = puzzle['name']
            rename_sheet(turb, spreadsheet_id, res['sheetId'], sheet_name)

    # Next, delete the blank sheet that's was created before the template
    sheet_id = spreadsheet['sheets'][0]['properties']['sheetId']
    delete_sheet(turb, spreadsheet_id, sheet_id)

    # Insert some useful links into the sheet where expected
    if 'url' in puzzle:
        url_link = puzzle['url']
        url_text = "Original puzzle link"
        spreadsheet_insert_data(
            turb, spreadsheet_id, "B1:B1",
            '=HYPERLINK("{}","{}")'.format(url_link, url_text))

    if 'channel_url' in puzzle:
        url_link = puzzle['channel_url']
        url_text = "Slack channel link"
        spreadsheet_insert_data(
            turb, spreadsheet_id, "B2:B2",
            '=HYPERLINK("{}","{}")'.format(url_link, url_text))

    return {
        'id': spreadsheet_id,
        'url': spreadsheet['spreadsheetUrl']
    }

def spreadsheet_insert_data(turb, spreadsheet_id, range, text):

    turb.sheets.values().append(
        spreadsheetId=spreadsheet_id,
        range=range,
        valueInputOption='USER_ENTERED',
        insertDataOption='INSERT_ROWS',
        body={
            'range': range,
            'values': [
                [text]
            ]
        }
    ).execute()

def delete_sheet(turb, spreadsheet_id, sheet_id):

    body = {
        'requests': [{
            'deleteSheet': {
                'sheetId': sheet_id,
            },
        }]
    }

    turb.sheets.batchUpdate(spreadsheetId=spreadsheet_id,
                            body=body
                            ).execute()

def rename_sheet(turb, spreadsheet_id, sheet_id, name):

    body = {
        'requests': [{
            'updateSheetProperties': {
                'properties': {
                    'sheetId': sheet_id,
                    'title': name
                },
                'fields': 'title'
            }
        }]
    }

    turb.sheets.batchUpdate(spreadsheetId=spreadsheet_id,
                            body=body
                            ).execute()

def spreadsheet_id_from_url(url):
    # Google sheet ids are between the /d/ and /edit in the url, like
    # https://docs.google.com/spreadsheets/d/1dxHBzjen...-LaXeVPrg/edit#gid=0
    start = url.find('/d/') + 3
    end = url.find('/edit')
    return url[start:end]

def rename_spreadsheet(turb, spreadsheet_url, name):

    spreadsheet_id = spreadsheet_id_from_url(spreadsheet_url)

    body = {
        'requests': [{
            'updateSpreadsheetProperties': {
                'properties': {
                    'title': name
                },
                'fields': 'title'
            }
        }]
    }

    turb.sheets.batchUpdate(spreadsheetId=spreadsheet_id,
                            body=body
                            ).execute()
